@@ -1,6 +1,7 @@
 import os
 import xml.etree.ElementTree as ET
-
+import time
+import datetime
 
 class SoftLinx:
     def __init__(
@@ -16,6 +17,7 @@ class SoftLinx:
         self.protocolSteps = []
         self.variables = []
         self.plates = {}
+        self.manifest_list = []  
         self.plugin_flags = {
             "PlateCrane": False,
             "Plates": False,
@@ -37,6 +39,7 @@ class SoftLinx:
             # "RapidPick": 8,
             # "TorreyPinesRIC20": 9,
         }
+        
 
         # *Set Protocol Name
         try:
@@ -112,6 +115,11 @@ class SoftLinx:
                 "Plates must be a dict with key 'position' and value 'plate name'."
             )
         self.plates = plates
+    
+    def addToManifest(self, filename):   
+        if not isinstance(filename, str):
+            raise TypeError("filename must be a string to add to manifest.")
+        self.manifest_list.append(filename)
 
     def activityCapacityCalculator(self):
         if len(self.protocolSteps) > 0:
@@ -262,7 +270,63 @@ class SoftLinx:
             file.write("\tSleep, 5000\n")
             file.write("}\n")
 
+    def generateConditionalXML(self, parent, step):
+        activity_dict = {
+            "Text": "{x:Null}",
+            "ActivityLabel": "",
+            "DisplayName": step["DisplayName"],
+            "HasConstraints": "False",
+            "SLXId": step["SLXId"],
+            "ToolTip": step["ToolTip"],
+            "UserComments": "",
+            "isActive": step["isActive"],
+            "isSetup": "True",
+        }
+        ifelse_xml = ET.SubElement(parent, "IfElseActivity", activity_dict)
+        ifelse_activities = ET.SubElement(ifelse_xml, "IfElseActivity.Activities")
+        scg_list = ET.SubElement(
+            ifelse_activities,
+            "scg:List",
+            {
+                "x:TypeArguments": "p:Activity",
+                "Capacity": "4",
+            },
+        )
+        for substep in step["branchTrue"]:
+            self.generateStepXML(scg_list, substep)
+        ifelse_activities2 = ET.SubElement(ifelse_xml, "IfElseActivity.Activities")
+        scg_list = ET.SubElement(
+            ifelse_activities2,
+            "scg:List",
+            {
+                "x:TypeArguments": "p:Activity",
+                "Capacity": "4",
+            },
+        )
+        for substep in step["branchFalse"]:
+            self.generateStepXML(scg_list, substep)
+        ifelse_arguments = ET.SubElement(ifelse_xml, "IfElseActivity.Arguments")
+        ET.SubElement(
+            ifelse_arguments,
+            "IfElseActivityArguments",
+            {"Condition": step["conditionalStatement"]},
+        )
+        ifelse_timeconstraints = ET.SubElement(
+            ifelse_xml, "IfElseActivity.TimeConstraints"
+        )
+        ET.SubElement(
+            ifelse_timeconstraints,
+            "scg:List",
+            {
+                "x:TypeArguments": "hwab:TimeConstraint",
+                "Capacity": "0",
+            },
+        )
+
     def generateStepXML(self, scg_list, step):
+        if step["type"] == "IfElseActivity":
+            self.generateConditionalXML(scg_list, step)
+            return
         self.plugin_flags[step["system"]] = True
         activity_dict = {
             "IconPath": "{x:Null}",
@@ -330,6 +394,37 @@ class SoftLinx:
         workflowViewState = ET.SubElement(step_xml, "sap2010:WorkflowViewState.IdRef")
         workflowViewState.text = "InstrumentActivity_1"
 
+    def conditional(
+        self,
+        conditionalStatement="",
+        branchTrue=[],
+        branchFalse=[],
+        isActive=True,
+        displayName="Conditional Statement",
+        index=None,
+        inplace=True,
+    ):
+        if not isinstance(branchTrue, list) or not isinstance(branchFalse, list):
+            raise ValueError("branchTrue and branchFalse must be a list of steps")
+        step = {
+            "type": "IfElseActivity",
+            "DisplayName": str(displayName),
+            "Command": "IfElseActivity",
+            "ToolTip": "If '%s'..." % (conditionalStatement),
+            "SLXId": "7744fd63-8699-40b9-9241-741e701dd8b3",
+            "isActive": str(isActive),
+            "branchTrue": branchTrue,
+            "branchFalse": branchFalse,
+            "conditionalStatement": str(conditionalStatement),
+        }
+
+        if inplace:
+            if index != None:
+                self.protocolSteps.insert(index, step)
+            else:
+                self.protocolSteps.append(step)
+        return step
+
     def plateCraneMoveCrane(
         self,
         position="SoftLinx.PlateCrane.Safe",
@@ -359,10 +454,65 @@ class SoftLinx:
         self,
         positionsFrom=["SoftLinx.PlateCrane.Stack1"],
         positionsTo=["SoftLinx.PlateCrane.Stack2"],
+        hasLid=False,
+        inGripper=False,
+        flip180=False,
+        onEmptyStack="End Method",
+        onMoveComplete="Move to Safe Location",
+        moveUp=999,
+        nestIsSpringLoaded=False,
+        checkForPlatesInAllPositions=False,
         isActive=True,
         index=None,
         inplace=True,
     ):
+        if inGripper:
+            positionsFrom = ["SoftLinx.PlateCrane.Gripper"]
+        onEmptyStackDict = {
+            "End Method": 0,
+            "Post Message and Continue": 1,
+            "Prompt User": 0,
+            "Complete Method": 256,
+            "Ignore and Continue": 512,
+        }
+        onMoveCompleteDict = {
+            "Move Up": "Up",
+            "Move to Safe Location": "Safe",
+            "Hold Plate in Nest": "Hold",
+            "Wait for Lid Removal": "Lid",
+        }
+        if isinstance(onEmptyStack, str):
+            try:
+                onEmptyStackVal = onEmptyStackDict[onEmptyStack]
+            except:
+                raise ValueError(
+                    "onEmptyStack must be one of the following:"
+                    + str([key + ", " for key in onEmptyStackDict])
+                )
+        else:
+            raise ValueError(
+                "onEmptyStack must be one of the following:"
+                + str([key + ", " for key in onEmptyStackDict])
+            )
+        if checkForPlatesInAllPositions:
+            onEmptyStackVal += 240
+        if isinstance(onMoveComplete, str):
+            try:
+                onMoveCompleteVal = onMoveCompleteDict[onMoveComplete]
+            except:
+                raise ValueError(
+                    "onMoveComplete must be one of the following:"
+                    + str([key + ", " for key in onMoveCompleteDict])
+                )
+        else:
+            raise ValueError(
+                "onMoveComplete must be one of the following:"
+                + str([key + ", " for key in onMoveCompleteDict])
+            )
+        if onMoveCompleteVal == "Up":
+            if not isinstance(moveUp, int):
+                raise ValueError("moveUp must be an integer")
+            onMoveCompleteVal += str(moveUp)
         step = {
             "type": "MovePlate",
             "Command": "Move Plate",
@@ -378,12 +528,12 @@ class SoftLinx:
             "args": [
                 ["x:String", ",".join(positionsFrom)],
                 ["x:String", ",".join(positionsTo)],
-                ["x:String", "Safe"],
-                ["x:String", "0"],
+                ["x:String", str(onMoveCompleteVal)],
+                ["x:String", str(onEmptyStackVal)],
                 ["x:String", " "],
-                ["x:String", "False"],
-                ["x:String", "False"],
-                ["x:String", "False"],
+                ["x:String", str(hasLid)],
+                ["x:String", str(nestIsSpringLoaded)],
+                ["x:String", str(flip180)],
                 ["x:String", " "],
             ],
         }
@@ -416,8 +566,37 @@ class SoftLinx:
                 self.protocolSteps.insert(index, step)
             else:
                 self.protocolSteps.append(step)
+        #* Add .hso filename to manifest list    
+        if "\\" in filename:      
+            hso_filename = filename.split("\\")[-1]
+        elif "/" in filename:
+            hso_filename = filename.split("/")[-1]
+        else:
+            hso_filename = filename
+        self.addToManifest(hso_filename)
+
         return step
 
+    def soloSoftResetTipCount(
+        self, position=1, isActive=True, index=None, inplace=True
+    ):
+        if not isinstance(position, int):
+            raise TypeError("position must be an integer")
+        if position < 1 or position > 8:
+            raise ValueError("position must be between 1 and 8, inclusive")
+        step = {
+            "type": "Reset Tip Count",
+            "Command": "Reset Tip Count",
+            "Description": "Position:		" + str(position),
+            "SLXId": "148788b4-63eb-4424-9af8-a175fb195afb",
+            "ToolTip": "Position:		" + str(position),
+            "isActive": str(isActive),
+            "system": "Solo",
+            "args": [
+                str(position),
+            ],
+        }
+
     def saveProtocol(self, filename=None, generate_ahk=True):
         if filename == None:
             if self.filename != None:
@@ -547,8 +726,17 @@ class SoftLinx:
         ).replace("&amp;", "&")
         with open(filename, "w") as file:
             file.write(xmlstring)
+        # *Add .slvp filename to manifest list
+        self.addToManifest(os.path.basename(filename))
         # *Generate AutoHotKey script
-        self.generateAutoHotKey(os.path.basename(filename), os.path.splitext(filename)[0] + ".ahk")
+        self.generateAutoHotKey(
+            os.path.basename(filename), os.path.splitext(filename)[0] + ".ahk"
+        )
+        # *Add AutoHotKey filename to manifest list
+        self.addToManifest(os.path.splitext(filename)[0] + ".ahk")
+        # *Generate Manifest File
+        self.generateManifest()   
+        #)
 
     # *Pretty-print XML
     def indent(self, element, level=0):
@@ -567,3 +755,14 @@ class SoftLinx:
             if level and (not element.tail or not element.tail.strip()):
                 element.tail = j
         return element
+
+    def generateManifest(self, manifest_filename = None):     
+        if manifest_filename == None:
+            manifest_filename = os.path.splitext(self.filename)[0] + ".txt"
+        else: 
+            manifest_filename = os.path.abspath(self.filename).replace(os.path.basename(self.filename), manifest_filename)
+
+        with open(manifest_filename, "w+") as manifest_file:
+            manifest_file.write(str(time.time()) + "\n" + str(datetime.datetime.now()) + "\n")  # add timestamps
+            manifest_file.writelines("\n".join(self.manifest_list))
+
